###################################################
### chunk number 1: 
###################################################
options(width = 75)
### for sampling
set.seed(1234)


###################################################
### chunk number 2: 
###################################################
library("TSP")
data("USCA50")
USCA50


###################################################
### chunk number 3: 
###################################################
set.seed(1234)


###################################################
### chunk number 4: dotchart_USCA50
###################################################
methods <- c("nearest_insertion", "farthest_insertion", "cheapest_insertion",
    "arbitrary_insertion", "nn", "repetitive_nn", "2-opt")

tours <- sapply(methods, FUN = function(m) solve_TSP(USCA50, method = m),
                simplify = FALSE)         
## names(tours) <- methods
## tours$concorde  <- solve_TSP(tsp, method = "concorde")
## tours$concorde  <- solve_TSP(tsp, method = "linkern")

tours[[1]]


## opt <- 14497
dotchart(c(sapply(tours, FUN = attr, "tour_length"), optimal = 14497), 
xlab = "tour length", xlim = c(0, 20000))
## add optimum
#abline(v = opt, col = "red")
#mtext("optimum", at = opt, side = 3, col = "red")


###################################################
### chunk number 5: 
###################################################
set.seed(1234)


###################################################
### chunk number 6: 
###################################################
library("TSP")
data("USCA312")
tsp <- insert_dummy(USCA312, label = "cut")
tsp


###################################################
### chunk number 7: 
###################################################
tour <- solve_TSP(tsp, method="farthest_insertion")
#tour <- solve_TSP(tsp, method ="concorde")
tour


###################################################
### chunk number 8: 
###################################################
path <- cut_tour(tour, "cut")
head(labels(path))
tail(labels(path))


###################################################
### chunk number 9: map1
###################################################
library("maps")
library("sp")
library("maptools")

data("USCA312_map")

plot_path <- function(path){
    plot(as(USCA312_coords, "Spatial"), axes = TRUE)
    plot(USCA312_basemap, add = TRUE, col = "gray")
    points(USCA312_coords, pch = 3, cex = 0.4, col = "red")
    
    path_line <- SpatialLines(list(Lines(list(Line(USCA312_coords[path,])))))
    plot(path_line, add=TRUE, col = "black")
    points(USCA312_coords[c(head(path,1), tail(path,1)),], pch = 19, 
        col = "black")
}

plot_path(path)


###################################################
### chunk number 10: 
###################################################
set.seed(1234)


###################################################
### chunk number 11: 
###################################################
atsp <- as.ATSP(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
atsp[, ny] <- 0
initial_tour <- solve_TSP(atsp, method="nn")
initial_tour
tour <- solve_TSP(atsp, method ="2-opt", control = list(tour = initial_tour))
tour
path <- cut_tour(tour, ny, exclude_cut = FALSE)

head(labels(path))
tail(labels(path))


###################################################
### chunk number 12: map2
###################################################
plot_path(path)


###################################################
### chunk number 13: 
###################################################
tsp <- reformulate_ATSP_as_TSP(atsp)
tsp


###################################################
### chunk number 14:  eval=FALSE
###################################################
## #tour <- solve_TSP(tsp, method = "linkern")
## tour <- solve_TSP(tsp, method = "concorde")
## tour <- as.TOUR(tour[tour <= n_of_cities(atsp)])


###################################################
### chunk number 15: 
###################################################
set.seed(1234)


###################################################
### chunk number 16: 
###################################################
m <- as.matrix(USCA312)
ny <- which(labels(USCA312) == "New York, NY")
la <- which(labels(USCA312) == "Los Angeles, CA")

atsp <- ATSP(m[-c(ny,la), -c(ny,la)])
atsp <- insert_dummy(atsp, label = "LA/NY")

la_ny <- which(labels(atsp) == "LA/NY")
atsp[la_ny, ] <- c(m[-c(ny,la), ny], 0)
atsp[, la_ny] <- c(m[la, -c(ny,la)], 0)


###################################################
### chunk number 17: 
###################################################
tour <- solve_TSP(atsp, method ="nearest_insertion")
tour

path_labels <- c("New York, NY", 
    labels(cut_tour(tour, la_ny)), "Los Angeles, CA")
path_ids <- match(path_labels, labels(USCA312))

head(path_labels)
tail(path_labels)


###################################################
### chunk number 18: map3
###################################################
plot_path(path_ids)


###################################################
### chunk number 19: 
###################################################
set.seed(4444)


###################################################
### chunk number 20: 
###################################################
data("iris")
tsp <- TSP(dist(iris[-5]), labels = iris[, "Species"])
#tsp <- TSP(dist(scale(iris[-5])))
tsp_dummy <- insert_dummy(tsp, n = 3, label = "boundary")
tour <- solve_TSP(tsp_dummy)


###################################################
### chunk number 21: clustering
###################################################
## plot the distance matrix
image(tsp_dummy, tour, xlab = "objects", ylab ="objects")

## draw lines where the dummy cities are located
abline(h = which(labels(tour)=="boundary"), col = "red")
abline(v = which(labels(tour)=="boundary"), col = "red")


###################################################
### chunk number 22: 
###################################################
## labels(tour)
#as.vector(abbreviate(labels(tour), dot = TRUE))
out <- rle(labels(tour))
data.frame(Species = out$values, 
           Lenghts = out$lengths, 
           Pos = cumsum(out$lengths))

## do Concorde
# tour <- solve_TSP(tsp_dummy, method = "concorde")
# labels(tour)


###################################################
### chunk number 23: clustering2
###################################################
prc <- prcomp(iris[1:4])
plot(prc$x, pch = as.numeric(iris[,5]), col =  as.numeric(iris[,5]))

indices <- c(tour, tour[1])
indices[indices > 150] <- NA
lines(prc$x[indices,])


